@@ -1,5 +1,5 @@
 /* engine-backend.h - A crypto backend for the engine interface.
-   Copyright (C) 2002, 2003, 2004 g10 Code GmbH
+   Copyright (C) 2002, 2003, 2004, 2009 g10 Code GmbH
  
    This file is part of GPGME.
  
@@ -14,20 +14,14 @@
    Lesser General Public License for more details.
    
    You should have received a copy of the GNU Lesser General Public
-   License along with this program; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
-   02111-1307, USA.  */
+   License along with this program; if not, see <http://www.gnu.org/licenses/>.
+ */
 
 #ifndef ENGINE_BACKEND_H
 #define ENGINE_BACKEND_H
 
 #include "engine.h"
 
-/* FIXME: Correct check?  */
-#ifdef GPGSM_PATH
-#define ENABLE_GPGSM 1
-#endif
-
 struct engine_ops
 {
   /* Static functions.  */
@@ -35,6 +29,11 @@ struct engine_ops
   /* Return the default file name for the binary of this engine.  */
   const char *(*get_file_name) (void);
 
+  /* Return the default home dir for the binary of this engine.  If
+     this function pointer is not set, the standard default home dir
+     of the engine is used. */
+  const char *(*get_home_dir) (void);
+
   /* Returns a malloced string containing the version of the engine
      with the given binary file name (or the default if FILE_NAME is
      NULL.  */
@@ -59,8 +58,11 @@ struct engine_ops
 					   engine_colon_line_handler_t fnc,
 					   void *fnc_value);
   gpgme_error_t (*set_locale) (void *engine, int category, const char *value);
+  gpgme_error_t (*set_protocol) (void *engine, gpgme_protocol_t protocol);
   gpgme_error_t (*decrypt) (void *engine, gpgme_data_t ciph,
 			    gpgme_data_t plain);
+  gpgme_error_t (*decrypt_verify) (void *engine, gpgme_data_t ciph,
+				   gpgme_data_t plain);
   gpgme_error_t (*delete) (void *engine, gpgme_key_t key, int allow_secret);
   gpgme_error_t (*edit) (void *engine, int type, gpgme_key_t key,
 			 gpgme_data_t out, gpgme_ctx_t ctx /* FIXME */);
@@ -73,14 +75,15 @@ struct engine_ops
 				 gpgme_data_t plain, gpgme_data_t ciph,
 				 int use_armor, gpgme_ctx_t ctx /* FIXME */);
   gpgme_error_t (*export) (void *engine, const char *pattern,
-			   unsigned int reserved, gpgme_data_t keydata,
+			   gpgme_export_mode_t mode, gpgme_data_t keydata,
 			   int use_armor);
   gpgme_error_t (*export_ext) (void *engine, const char *pattern[],
-			       unsigned int reserved, gpgme_data_t keydata,
+			       gpgme_export_mode_t mode, gpgme_data_t keydata,
 			       int use_armor);
   gpgme_error_t (*genkey) (void *engine, gpgme_data_t help_data, int use_armor,
 			   gpgme_data_t pubkey, gpgme_data_t seckey);
-  gpgme_error_t (*import) (void *engine, gpgme_data_t keydata);
+  gpgme_error_t (*import) (void *engine, gpgme_data_t keydata,
+                           gpgme_key_t *keyarray);
   gpgme_error_t (*keylist) (void *engine, const char *pattern,
 			    int secret_only, gpgme_keylist_mode_t mode);
   gpgme_error_t (*keylist_ext) (void *engine, const char *pattern[],
@@ -88,14 +91,21 @@ struct engine_ops
 				gpgme_keylist_mode_t mode);
   gpgme_error_t (*sign) (void *engine, gpgme_data_t in, gpgme_data_t out,
 			 gpgme_sig_mode_t mode, int use_armor,
-			 int use_textmode,
-			 int include_certs, gpgme_ctx_t ctx /* FIXME */);
+			 int use_textmode, int include_certs,
+			 gpgme_ctx_t ctx /* FIXME */);
   gpgme_error_t (*trustlist) (void *engine, const char *pattern);
   gpgme_error_t (*verify) (void *engine, gpgme_data_t sig,
-			   gpgme_data_t signed_text,
-			   gpgme_data_t plaintext);
+			   gpgme_data_t signed_text, gpgme_data_t plaintext);
   gpgme_error_t  (*getauditlog) (void *engine, gpgme_data_t output,
                                  unsigned int flags);
+  gpgme_error_t  (*opassuan_transact) (void *engine, 
+                                       const char *command,
+                                       gpgme_assuan_data_cb_t data_cb,
+                                       void *data_cb_value,
+                                       gpgme_assuan_inquire_cb_t inq_cb,
+                                       void *inq_cb_value,
+                                       gpgme_assuan_status_cb_t status_cb,
+                                       void *status_cb_value);
 
   gpgme_error_t  (*conf_load) (void *engine, gpgme_conf_comp_t *conf_p);
   gpgme_error_t  (*conf_save) (void *engine, gpgme_conf_comp_t conf);
@@ -103,7 +113,14 @@ struct engine_ops
   void (*set_io_cbs) (void *engine, gpgme_io_cbs_t io_cbs);
   void (*io_event) (void *engine, gpgme_event_io_t type, void *type_data);
 
+  /* Cancel the whole engine session.  */
   gpgme_error_t (*cancel) (void *engine);
+
+  /* Cancel only the current operation, not the whole session.  */
+  gpgme_error_t (*cancel_op) (void *engine);
+
+  /* Change the passphrase for KEY. */
+  gpgme_error_t (*passwd) (void *engine, gpgme_key_t key, unsigned int flags);
 };
 
 
@@ -114,5 +131,14 @@ extern struct engine_ops _gpgme_engine_ops_gpgsm;	/* CMS.  */
 #ifdef ENABLE_GPGCONF
 extern struct engine_ops _gpgme_engine_ops_gpgconf;	/* gpg-conf.  */
 #endif
+#ifdef ENABLE_ASSUAN
+extern struct engine_ops _gpgme_engine_ops_assuan;	/* Low-level Assuan. */
+#endif
+#ifdef ENABLE_G13
+extern struct engine_ops _gpgme_engine_ops_g13;         /* Crypto VFS. */
+#endif
+#ifdef ENABLE_UISERVER
+extern struct engine_ops _gpgme_engine_ops_uiserver;
+#endif
 
 #endif /* ENGINE_BACKEND_H */
