@@ -95,6 +95,9 @@ gpgme_op_assuan_transact_start (gpgme_ctx_t ctx,
 	      command, data_cb, data_cb_value, inq_cb, inq_cb_value,
 	      status_cb, status_cb_value);
 
+  if (!ctx)
+    return TRACE_ERR (gpg_error (GPG_ERR_INV_VALUE));
+
   err = opassuan_start (ctx, 0, command, data_cb, data_cb_value,
 			inq_cb, inq_cb_value, status_cb, status_cb_value);
   return TRACE_ERR (err);
@@ -122,13 +125,17 @@ gpgme_op_assuan_transact_ext (gpgme_ctx_t ctx,
 	      command, data_cb, data_cb_value, inq_cb, inq_cb_value,
 	      status_cb, status_cb_value, op_err_p);
 
+  if (!ctx)
+    return TRACE_ERR (gpg_error (GPG_ERR_INV_VALUE));
+
   err = opassuan_start (ctx, 1, command, 
                         data_cb, data_cb_value,
                         inq_cb, inq_cb_value,
                         status_cb, status_cb_value);
-  if (!err)
-    err = _gpgme_wait_one_ext (ctx, &op_err);
+  if (err)
+    goto out;
 
+  err = _gpgme_wait_one_ext (ctx, &op_err);
   if (op_err)
     {
       TRACE_LOG2 ("op_err = %s <%s>", gpgme_strerror (op_err),
@@ -140,7 +147,8 @@ gpgme_op_assuan_transact_ext (gpgme_ctx_t ctx,
     }
   if (op_err_p)
     *op_err_p = op_err;
-
+  
+ out:
   return TRACE_ERR (err);
 }
 
@@ -210,6 +218,9 @@ gpgme_op_assuan_transact (gpgme_ctx_t ctx,
 
   TRACE (DEBUG_CTX, "gpgme_op_assuan_transact", ctx);
 
+  if (!ctx)
+    return gpg_error (GPG_ERR_INV_VALUE);
+
   /* Users of the old-style session based interfaces need to look at
      the result structure.  */
   err = gpgme_op_assuan_transact_ext (ctx, command, data_cb, data_cb_value,
