@@ -3,17 +3,17 @@
    Copyright (C) 2001, 2002, 2003, 2004, 2007, 2010 g10 Code GmbH
 
    This file is part of GPGME.
-
+ 
    GPGME is free software; you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation; either version 2.1 of
    the License, or (at your option) any later version.
-
+   
    GPGME is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.
-
+   
    You should have received a copy of the GNU Lesser General Public
    License along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
@@ -82,7 +82,7 @@ static struct
      want to close something.  Using the same handle for these
      duplicates works just fine.  */
   int dup_from;
-} fd_table[MAX_SLAFD];
+} fd_table[MAX_SLAFD];  
 
 
 /* Returns the FD or -1 on resource limit.  */
@@ -147,7 +147,7 @@ struct reader_context_s
 {
   HANDLE file_hd;
   int file_sock;
-  HANDLE thread_hd;
+  HANDLE thread_hd;	
   int refcount;
 
   DECLARE_LOCK (mutex);
@@ -157,14 +157,12 @@ struct reader_context_s
   int eof_shortcut;
   int error;
   int error_code;
-
+  
   /* This is manually reset.  */
   HANDLE have_data_ev;
   /* This is automatically reset.  */
   HANDLE have_space_ev;
-  /* This is manually reset but actually only triggered once.  */
-  HANDLE close_ev;
-
+  HANDLE stopped;
   size_t readpos, writepos;
   char buffer[READBUF_SIZE];
 };
@@ -184,11 +182,11 @@ struct writer_context_s
 {
   HANDLE file_hd;
   int file_sock;
-  HANDLE thread_hd;
+  HANDLE thread_hd;	
   int refcount;
 
   DECLARE_LOCK (mutex);
-
+  
   int stop_me;
   int error;
   int error_code;
@@ -196,8 +194,8 @@ struct writer_context_s
   /* This is manually reset.  */
   HANDLE have_data;
   HANDLE is_empty;
-  HANDLE close_ev;
-  size_t nbytes;
+  HANDLE stopped;
+  size_t nbytes; 
   char buffer[WRITEBUF_SIZE];
 };
 
@@ -259,15 +257,15 @@ set_synchronize (HANDLE hd)
 }
 
 
-static DWORD CALLBACK
+static DWORD CALLBACK 
 reader (void *arg)
 {
   struct reader_context_s *ctx = arg;
   int nbytes;
   DWORD nread;
   int sock;
-  TRACE_BEG2 (DEBUG_SYSIO, "gpgme:reader", ctx->file_hd,
-	      "file_sock=%d, thread=%p", ctx->file_sock, ctx->thread_hd);
+  TRACE_BEG1 (DEBUG_SYSIO, "gpgme:reader", ctx->file_hd,
+	      "thread=%p", ctx->thread_hd);
 
   if (ctx->file_hd != INVALID_HANDLE_VALUE)
     sock = 0;
@@ -280,7 +278,7 @@ reader (void *arg)
       /* Leave a 1 byte gap so that we can see whether it is empty or
 	 full.  */
       if ((ctx->writepos + 1) % READBUF_SIZE == ctx->readpos)
-	{
+	{ 
 	  /* Wait for space.  */
 	  if (!ResetEvent (ctx->have_space_ev))
 	    TRACE_LOG1 ("ResetEvent failed: ec=%d", (int) GetLastError ());
@@ -300,7 +298,7 @@ reader (void *arg)
       if (nbytes > READBUF_SIZE - ctx->writepos)
 	nbytes = READBUF_SIZE - ctx->writepos;
       UNLOCK (ctx->mutex);
-
+      
       TRACE_LOG2 ("%s %d bytes", sock? "receiving":"reading", nbytes);
 
       if (sock)
@@ -318,21 +316,6 @@ reader (void *arg)
                 }
               else
                 {
-                  /* Check whether the shutdown triggered the error -
-                     no need to to print a warning in this case.  */
-                  if ( ctx->error_code == WSAECONNABORTED
-                       || ctx->error_code == WSAECONNRESET)
-                    {
-                      LOCK (ctx->mutex);
-                      if (ctx->stop_me)
-                        {
-                          UNLOCK (ctx->mutex);
-                          TRACE_LOG ("got shutdown");
-                          break;
-                        }
-                      UNLOCK (ctx->mutex);
-                    }
-
                   ctx->error = 1;
                   TRACE_LOG1 ("recv error: ec=%d", ctx->error_code);
                 }
@@ -374,9 +357,8 @@ reader (void *arg)
 	  UNLOCK (ctx->mutex);
 	  break;
         }
-
       TRACE_LOG1 ("got %u bytes", nread);
-
+      
       ctx->writepos = (ctx->writepos + nread) % READBUF_SIZE;
       if (!SetEvent (ctx->have_data_ev))
 	TRACE_LOG2 ("SetEvent (0x%x) failed: ec=%d", ctx->have_data_ev,
@@ -385,19 +367,10 @@ reader (void *arg)
     }
   /* Indicate that we have an error or EOF.  */
   if (!SetEvent (ctx->have_data_ev))
-    TRACE_LOG2 ("SetEvent (0x%x) failed: ec=%d", ctx->have_data_ev,
-                (int) GetLastError ());
-
-  TRACE_LOG ("waiting for close");
-  WaitForSingleObject (ctx->close_ev, INFINITE);
-
-  CloseHandle (ctx->close_ev);
-  CloseHandle (ctx->have_data_ev);
-  CloseHandle (ctx->have_space_ev);
-  CloseHandle (ctx->thread_hd);
-  DESTROY_LOCK (ctx->mutex);
-  free (ctx);
-
+	TRACE_LOG2 ("SetEvent (0x%x) failed: ec=%d", ctx->have_data_ev,
+		    (int) GetLastError ());
+  SetEvent (ctx->stopped);
+  
   return TRACE_SUC ();
 }
 
@@ -414,7 +387,7 @@ create_reader (int fd)
   memset (&sec_attr, 0, sizeof sec_attr);
   sec_attr.nLength = sizeof sec_attr;
   sec_attr.bInheritHandle = FALSE;
-
+  
   ctx = calloc (1, sizeof *ctx);
   if (!ctx)
     {
@@ -425,12 +398,8 @@ create_reader (int fd)
   if (fd < 0 || fd >= MAX_SLAFD || !fd_table[fd].used)
     {
       TRACE_SYSERR (EIO);
-      free (ctx);
       return NULL;
     }
-  TRACE_LOG4 ("fd=%d -> handle=%p socket=%d dupfrom=%d",
-              fd, fd_table[fd].handle, fd_table[fd].socket,
-              fd_table[fd].dup_from);
   ctx->file_hd = fd_table[fd].handle;
   ctx->file_sock = fd_table[fd].socket;
 
@@ -439,16 +408,16 @@ create_reader (int fd)
   if (ctx->have_data_ev)
     ctx->have_space_ev = CreateEvent (&sec_attr, FALSE, TRUE, NULL);
   if (ctx->have_space_ev)
-    ctx->close_ev = CreateEvent (&sec_attr, TRUE, FALSE, NULL);
-  if (!ctx->have_data_ev || !ctx->have_space_ev || !ctx->close_ev)
+    ctx->stopped = CreateEvent (&sec_attr, TRUE, FALSE, NULL);
+  if (!ctx->have_data_ev || !ctx->have_space_ev || !ctx->stopped)
     {
       TRACE_LOG1 ("CreateEvent failed: ec=%d", (int) GetLastError ());
       if (ctx->have_data_ev)
 	CloseHandle (ctx->have_data_ev);
       if (ctx->have_space_ev)
 	CloseHandle (ctx->have_space_ev);
-      if (ctx->close_ev)
-	CloseHandle (ctx->close_ev);
+      if (ctx->stopped)
+	CloseHandle (ctx->stopped);
       free (ctx);
       /* FIXME: Translate the error code.  */
       TRACE_SYSERR (EIO);
@@ -473,12 +442,12 @@ create_reader (int fd)
 	CloseHandle (ctx->have_data_ev);
       if (ctx->have_space_ev)
 	CloseHandle (ctx->have_space_ev);
-      if (ctx->close_ev)
-	CloseHandle (ctx->close_ev);
+      if (ctx->stopped)
+	CloseHandle (ctx->stopped);
       free (ctx);
       TRACE_SYSERR (EIO);
       return NULL;
-    }
+    }    
   else
     {
       /* We set the priority of the thread higher because we know that
@@ -492,9 +461,6 @@ create_reader (int fd)
 }
 
 
-/* Prepare destruction of the reader thread for CTX.  Returns 0 if a
-   call to this function is sufficient and destroy_reader_finish shall
-   not be called.  */
 static void
 destroy_reader (struct reader_context_s *ctx)
 {
@@ -506,7 +472,7 @@ destroy_reader (struct reader_context_s *ctx)
       return;
     }
   ctx->stop_me = 1;
-  if (ctx->have_space_ev)
+  if (ctx->have_space_ev) 
     SetEvent (ctx->have_space_ev);
   UNLOCK (ctx->mutex);
 
@@ -526,32 +492,24 @@ destroy_reader (struct reader_context_s *ctx)
     }
 #endif
 
-  /* The reader thread is usually blocking in recv or ReadFile.  If
-     the peer does not send an EOF or breaks the pipe the WFSO might
-     get stuck waiting for the termination of the reader thread.  This
-     happens quite often with sockets, thus we definitely need to get
-     out of the recv.  A shutdown does this nicely.  For handles
-     (i.e. pipes) it would also be nice to cancel the operation, but
-     such a feature is only available since Vista.  Thus we need to
-     dlopen that syscall.  */
-  if (ctx->file_hd != INVALID_HANDLE_VALUE)
-    {
-      /* Fixme: Call CancelSynchronousIo (handle_of_thread).  */
-    }
-  else if (ctx->file_sock != INVALID_SOCKET)
-    {
-      if (shutdown (ctx->file_sock, 2))
-        TRACE2 (DEBUG_SYSIO, "gpgme:destroy_reader", ctx->file_hd,
-                "shutdown socket %d failed: %s",
-                ctx->file_sock, (int) WSAGetLastError ());
-    }
-
-  /* After setting this event CTX is void. */
-  SetEvent (ctx->close_ev);
+  TRACE1 (DEBUG_SYSIO, "gpgme:destroy_reader", ctx->file_hd,
+	  "waiting for termination of thread %p", ctx->thread_hd);
+  WaitForSingleObject (ctx->stopped, INFINITE);
+  TRACE1 (DEBUG_SYSIO, "gpgme:destroy_reader", ctx->file_hd,
+	  "thread %p has terminated", ctx->thread_hd);
+    
+  if (ctx->stopped)
+    CloseHandle (ctx->stopped);
+  if (ctx->have_data_ev)
+    CloseHandle (ctx->have_data_ev);
+  if (ctx->have_space_ev)
+    CloseHandle (ctx->have_space_ev);
+  CloseHandle (ctx->thread_hd);
+  DESTROY_LOCK (ctx->mutex);
+  free (ctx);
 }
 
 
-
 /* Find a reader context or create a new one.  Note that the reader
    context will last until a _gpgme_io_close.  */
 static struct reader_context_s *
@@ -588,6 +546,26 @@ find_reader (int fd, int start_it)
 }
 
 
+static void
+kill_reader (int fd)
+{
+  int i;
+
+  LOCK (reader_table_lock);
+  for (i = 0; i < reader_table_size; i++)
+    {
+      if (reader_table[i].used && reader_table[i].fd == fd)
+	{
+	  destroy_reader (reader_table[i].context);
+	  reader_table[i].context = NULL;
+	  reader_table[i].used = 0;
+	  break;
+	}
+    }
+  UNLOCK (reader_table_lock);
+}
+
+
 int
 _gpgme_io_read (int fd, void *buffer, size_t count)
 {
@@ -595,7 +573,7 @@ _gpgme_io_read (int fd, void *buffer, size_t count)
   struct reader_context_s *ctx;
   TRACE_BEG2 (DEBUG_SYSIO, "_gpgme_io_read", fd,
 	      "buffer=%p, count=%u", buffer, count);
-
+  
   ctx = find_reader (fd, 1);
   if (!ctx)
     {
@@ -615,7 +593,7 @@ _gpgme_io_read (int fd, void *buffer, size_t count)
       TRACE_LOG1 ("data from thread %p available", ctx->thread_hd);
       LOCK (ctx->mutex);
     }
-
+  
   if (ctx->readpos == ctx->writepos || ctx->error)
     {
       UNLOCK (ctx->mutex);
@@ -630,7 +608,7 @@ _gpgme_io_read (int fd, void *buffer, size_t count)
       gpg_err_set_errno (ctx->error_code);
       return TRACE_SYSRES (-1);
     }
-
+  
   nread = ctx->readpos < ctx->writepos
     ? ctx->writepos - ctx->readpos
     : READBUF_SIZE - ctx->readpos;
@@ -659,7 +637,7 @@ _gpgme_io_read (int fd, void *buffer, size_t count)
       return TRACE_SYSRES (-1);
     }
   UNLOCK (ctx->mutex);
-
+  
   TRACE_LOGBUF (buffer, nread);
   return TRACE_SYSRES (nread);
 }
@@ -668,14 +646,14 @@ _gpgme_io_read (int fd, void *buffer, size_t count)
 /* The writer does use a simple buffering strategy so that we are
    informed about write errors as soon as possible (i. e. with the the
    next call to the write function.  */
-static DWORD CALLBACK
+static DWORD CALLBACK 
 writer (void *arg)
 {
   struct writer_context_s *ctx = arg;
   DWORD nwritten;
   int sock;
-  TRACE_BEG2 (DEBUG_SYSIO, "gpgme:writer", ctx->file_hd,
-	      "file_sock=%d, thread=%p", ctx->file_sock, ctx->thread_hd);
+  TRACE_BEG1 (DEBUG_SYSIO, "gpgme:writer", ctx->file_hd,
+	      "thread=%p", ctx->thread_hd);
 
   if (ctx->file_hd != INVALID_HANDLE_VALUE)
     sock = 0;
@@ -691,7 +669,7 @@ writer (void *arg)
 	  break;
         }
       if (!ctx->nbytes)
-	{
+	{ 
 	  if (!SetEvent (ctx->is_empty))
 	    TRACE_LOG1 ("SetEvent failed: ec=%d", (int) GetLastError ());
 	  if (!ResetEvent (ctx->have_data))
@@ -708,7 +686,7 @@ writer (void *arg)
 	  break;
         }
       UNLOCK (ctx->mutex);
-
+      
       TRACE_LOG2 ("%s %d bytes", sock?"sending":"writing", ctx->nbytes);
 
       /* Note that CTX->nbytes is not zero at this point, because
@@ -749,7 +727,7 @@ writer (void *arg)
             }
         }
       TRACE_LOG1 ("wrote %d bytes", (int) nwritten);
-
+      
       LOCK (ctx->mutex);
       ctx->nbytes -= nwritten;
       UNLOCK (ctx->mutex);
@@ -757,16 +735,7 @@ writer (void *arg)
   /* Indicate that we have an error.  */
   if (!SetEvent (ctx->is_empty))
     TRACE_LOG1 ("SetEvent failed: ec=%d", (int) GetLastError ());
-
-  TRACE_LOG ("waiting for close");
-  WaitForSingleObject (ctx->close_ev, INFINITE);
-
-  CloseHandle (ctx->close_ev);
-  CloseHandle (ctx->have_data);
-  CloseHandle (ctx->is_empty);
-  CloseHandle (ctx->thread_hd);
-  DESTROY_LOCK (ctx->mutex);
-  free (ctx);
+  SetEvent (ctx->stopped);
 
   return TRACE_SUC ();
 }
@@ -791,16 +760,12 @@ create_writer (int fd)
       TRACE_SYSERR (errno);
       return NULL;
     }
-
+  
   if (fd < 0 || fd >= MAX_SLAFD || !fd_table[fd].used)
     {
       TRACE_SYSERR (EIO);
-      free (ctx);
       return NULL;
     }
-  TRACE_LOG4 ("fd=%d -> handle=%p socket=%d dupfrom=%d",
-              fd, fd_table[fd].handle, fd_table[fd].socket,
-              fd_table[fd].dup_from);
   ctx->file_hd = fd_table[fd].handle;
   ctx->file_sock = fd_table[fd].socket;
 
@@ -809,16 +774,16 @@ create_writer (int fd)
   if (ctx->have_data)
     ctx->is_empty  = CreateEvent (&sec_attr, TRUE, TRUE, NULL);
   if (ctx->is_empty)
-    ctx->close_ev = CreateEvent (&sec_attr, TRUE, FALSE, NULL);
-  if (!ctx->have_data || !ctx->is_empty || !ctx->close_ev)
+    ctx->stopped = CreateEvent (&sec_attr, TRUE, FALSE, NULL);
+  if (!ctx->have_data || !ctx->is_empty || !ctx->stopped)
     {
       TRACE_LOG1 ("CreateEvent failed: ec=%d", (int) GetLastError ());
       if (ctx->have_data)
 	CloseHandle (ctx->have_data);
       if (ctx->is_empty)
 	CloseHandle (ctx->is_empty);
-      if (ctx->close_ev)
-	CloseHandle (ctx->close_ev);
+      if (ctx->stopped)
+	CloseHandle (ctx->stopped);
       free (ctx);
       /* FIXME: Translate the error code.  */
       TRACE_SYSERR (EIO);
@@ -843,12 +808,12 @@ create_writer (int fd)
 	CloseHandle (ctx->have_data);
       if (ctx->is_empty)
 	CloseHandle (ctx->is_empty);
-      if (ctx->close_ev)
-	CloseHandle (ctx->close_ev);
+      if (ctx->stopped)
+	CloseHandle (ctx->stopped);
       free (ctx);
       TRACE_SYSERR (EIO);
       return NULL;
-    }
+    }    
   else
     {
       /* We set the priority of the thread higher because we know
@@ -861,7 +826,6 @@ create_writer (int fd)
   return ctx;
 }
 
-
 static void
 destroy_writer (struct writer_context_s *ctx)
 {
@@ -873,7 +837,7 @@ destroy_writer (struct writer_context_s *ctx)
       return;
     }
   ctx->stop_me = 1;
-  if (ctx->have_data)
+  if (ctx->have_data) 
     SetEvent (ctx->have_data);
   UNLOCK (ctx->mutex);
 
@@ -890,9 +854,22 @@ destroy_writer (struct writer_context_s *ctx)
 	      "unblock control call failed for thread %p", ctx->thread_hd);
     }
 #endif
-
-  /* After setting this event CTX is void.  */
-  SetEvent (ctx->close_ev);
+  
+  TRACE1 (DEBUG_SYSIO, "gpgme:destroy_writer", ctx->file_hd,
+	  "waiting for termination of thread %p", ctx->thread_hd);
+  WaitForSingleObject (ctx->stopped, INFINITE);
+  TRACE1 (DEBUG_SYSIO, "gpgme:destroy_writer", ctx->file_hd,
+	  "thread %p has terminated", ctx->thread_hd);
+  
+  if (ctx->stopped)
+    CloseHandle (ctx->stopped);
+  if (ctx->have_data)
+    CloseHandle (ctx->have_data);
+  if (ctx->is_empty)
+    CloseHandle (ctx->is_empty);
+  CloseHandle (ctx->thread_hd);
+  DESTROY_LOCK (ctx->mutex);
+  free (ctx);
 }
 
 
@@ -923,7 +900,7 @@ find_writer (int fd, int start_it)
     {
       wt = create_writer (fd);
       writer_table[i].fd = fd;
-      writer_table[i].context = wt;
+      writer_table[i].context = wt; 
       writer_table[i].used = 1;
     }
 
@@ -932,6 +909,26 @@ find_writer (int fd, int start_it)
 }
 
 
+static void
+kill_writer (int fd)
+{
+  int i;
+
+  LOCK (writer_table_lock);
+  for (i = 0; i < writer_table_size; i++)
+    {
+      if (writer_table[i].used && writer_table[i].fd == fd)
+	{
+	  destroy_writer (writer_table[i].context);
+	  writer_table[i].context = NULL;
+	  writer_table[i].used = 0;
+	  break;
+	}
+    }
+  UNLOCK (writer_table_lock);
+}
+
+
 int
 _gpgme_io_write (int fd, const void *buffer, size_t count)
 {
@@ -1061,14 +1058,14 @@ _gpgme_io_pipe (int filedes[2], int inherit_idx)
     {
       fd_table[rfd].handle = hd;
       fd_table[wfd].rvid = rvid;
-    }
+    }  
 
 #else
 
   memset (&sec_attr, 0, sizeof (sec_attr));
   sec_attr.nLength = sizeof (sec_attr);
   sec_attr.bInheritHandle = FALSE;
-
+  
   if (!CreatePipe (&rh, &wh, &sec_attr, PIPEBUF_SIZE))
     {
       TRACE_LOG1 ("CreatePipe failed: ec=%d", (int) GetLastError ());
@@ -1152,36 +1149,8 @@ _gpgme_io_close (int fd)
       return TRACE_SYSRES (-1);
     }
 
-  TRACE_LOG4 ("fd=%d -> handle=%p socket=%d dupfrom=%d",
-              fd, fd_table[fd].handle, fd_table[fd].socket,
-              fd_table[fd].dup_from);
-
-  LOCK (reader_table_lock);
-  for (i = 0; i < reader_table_size; i++)
-    {
-      if (reader_table[i].used && reader_table[i].fd == fd)
-	{
-	  destroy_reader (reader_table[i].context);
-	  reader_table[i].context = NULL;
-	  reader_table[i].used = 0;
-	  break;
-	}
-    }
-  UNLOCK (reader_table_lock);
-
-  LOCK (writer_table_lock);
-  for (i = 0; i < writer_table_size; i++)
-    {
-      if (writer_table[i].used && writer_table[i].fd == fd)
-	{
-	  destroy_writer (writer_table[i].context);
-	  writer_table[i].context = NULL;
-	  writer_table[i].used = 0;
-	  break;
-	}
-    }
-  UNLOCK (writer_table_lock);
-
+  kill_reader (fd);
+  kill_writer (fd);
   LOCK (notify_table_lock);
   for (i = 0; i < DIM (notify_table); i++)
     {
@@ -1204,7 +1173,7 @@ _gpgme_io_close (int fd)
       if (fd_table[fd].handle != INVALID_HANDLE_VALUE)
 	{
 	  if (!CloseHandle (fd_table[fd].handle))
-	    {
+	    { 
 	      TRACE_LOG1 ("CloseHandle failed: ec=%d", (int) GetLastError ());
 	      /* FIXME: Should translate the error code.  */
 	      gpg_err_set_errno (EIO);
@@ -1214,7 +1183,7 @@ _gpgme_io_close (int fd)
       else if (fd_table[fd].socket != INVALID_SOCKET)
 	{
 	  if (closesocket (fd_table[fd].socket))
-	    {
+	    { 
 	      TRACE_LOG1 ("closesocket failed: ec=%d", (int) WSAGetLastError ());
 	      /* FIXME: Should translate the error code.  */
 	      gpg_err_set_errno (EIO);
@@ -1225,7 +1194,7 @@ _gpgme_io_close (int fd)
     }
 
   release_fd (fd);
-
+      
   return TRACE_SYSRES (0);
 }
 
@@ -1284,7 +1253,7 @@ build_commandline (char **argv, int fd0, int fd0_isnull,
 
   p = fdbuf;
   *p = 0;
-
+  
   if (fd0 != -1)
     {
       if (fd0_isnull)
@@ -1311,7 +1280,7 @@ build_commandline (char **argv, int fd0, int fd0_isnull,
     }
   strcpy (p, "-&S2=null ");
   p += strlen (p);
-
+  
   n = strlen (fdbuf);
   for (i=0; (s = argv[i]); i++)
     {
@@ -1328,7 +1297,7 @@ build_commandline (char **argv, int fd0, int fd0_isnull,
     return NULL;
 
   p = stpcpy (p, fdbuf);
-  for (i = 0; argv[i]; i++)
+  for (i = 0; argv[i]; i++) 
     {
       if (!i)
         continue; /* Ignore argv[0].  */
@@ -1353,7 +1322,7 @@ build_commandline (char **argv, int fd0, int fd0_isnull,
         p = stpcpy (p, argv[i]);
     }
 
-  return buf;
+  return buf;  
 }
 #else
 static char *
@@ -1363,7 +1332,7 @@ build_commandline (char **argv)
   int n = 0;
   char *buf;
   char *p;
-
+  
   /* We have to quote some things because under Windows the program
      parses the commandline and does some unquoting.  We enclose the
      whole argument in double-quotes, and escape literal double-quotes
@@ -1463,7 +1432,7 @@ _gpgme_io_spawn (const char *path, char *const argv[], unsigned int flags,
 	  gpg_err_set_errno (EBADF);
 	  return TRACE_SYSRES (-1);
 	}
-
+      
       if (fd_list[i].dup_to == 0)
 	{
 	  fd_in = fd_list[i].fd;
@@ -1575,13 +1544,13 @@ _gpgme_io_spawn (const char *path, char *const argv[], unsigned int flags,
   args = calloc (2 + i + 1, sizeof (*args));
   args[0] = (char *) _gpgme_get_w32spawn_path ();
   args[1] = tmp_name;
-  args[2] = (char *)path;
+  args[2] = path;
   memcpy (&args[3], &argv[1], i * sizeof (*args));
 
   memset (&sec_attr, 0, sizeof sec_attr);
   sec_attr.nLength = sizeof sec_attr;
   sec_attr.bInheritHandle = FALSE;
-
+ 
   arg_string = build_commandline (args);
   free (args);
   if (!arg_string)
@@ -1600,8 +1569,7 @@ _gpgme_io_spawn (const char *path, char *const argv[], unsigned int flags,
   si.hStdError = INVALID_HANDLE_VALUE;
 
   cr_flags |= CREATE_SUSPENDED;
-  if ((flags & IOSPAWN_FLAG_DETACHED))
-    cr_flags |= DETACHED_PROCESS;
+  cr_flags |= DETACHED_PROCESS;
   cr_flags |= GetPriorityClass (GetCurrentProcess ());
   if (!CreateProcessA (_gpgme_get_w32spawn_path (),
 		       arg_string,
@@ -1635,7 +1603,7 @@ _gpgme_io_spawn (const char *path, char *const argv[], unsigned int flags,
       int fd = fd_list[i].fd;
       HANDLE ohd = INVALID_HANDLE_VALUE;
       HANDLE hd = INVALID_HANDLE_VALUE;
-
+      
       /* Make it inheritable for the wrapper process.  */
       if (fd >= 0 && fd < MAX_SLAFD && fd_table[fd].used)
 	ohd = fd_table[fd].handle;
@@ -1661,7 +1629,7 @@ _gpgme_io_spawn (const char *path, char *const argv[], unsigned int flags,
       /* Return the child name of this handle.  */
       fd_list[i].peer_name = handle_to_fd (hd);
     }
-
+  
   /* Write the handle translation information to the temporary
      file.  */
   {
@@ -1683,7 +1651,7 @@ _gpgme_io_spawn (const char *path, char *const argv[], unsigned int flags,
       {
 	/* Strip the newline.  */
 	len = strlen (line) - 1;
-
+	
 	/* Format is: Local name, stdin/stdout/stderr, peer name, argv idx.  */
 	snprintf (&line[len], BUFFER_MAX - len, "0x%x %d 0x%x %d  \n",
 		  fd_list[i].fd, fd_list[i].dup_to,
@@ -1710,16 +1678,16 @@ _gpgme_io_spawn (const char *path, char *const argv[], unsigned int flags,
 
   TRACE_LOG4 ("CreateProcess ready: hProcess=%p, hThread=%p, "
 	      "dwProcessID=%d, dwThreadId=%d",
-	      pi.hProcess, pi.hThread,
+	      pi.hProcess, pi.hThread, 
 	      (int) pi.dwProcessId, (int) pi.dwThreadId);
-
+  
   if (r_pid)
     *r_pid = (pid_t)pi.dwProcessId;
 
-
+  
   if (ResumeThread (pi.hThread) < 0)
     TRACE_LOG1 ("ResumeThread failed: ec=%d", (int) GetLastError ());
-
+  
   if (!CloseHandle (pi.hThread))
     TRACE_LOG1 ("CloseHandle of thread failed: ec=%d",
 		(int) GetLastError ());
@@ -1766,9 +1734,7 @@ _gpgme_io_select (struct io_select_fd_s *fds, size_t nfds, int nonblock)
   TRACE_BEG2 (DEBUG_SYSIO, "_gpgme_io_select", fds,
 	      "nfds=%u, nonblock=%u", nfds, nonblock);
 
-#if 0
  restart:
-#endif
   TRACE_SEQ (dbg_help, "select on [ ");
   any = 0;
   nwait = 0;
@@ -1783,7 +1749,7 @@ _gpgme_io_select (struct io_select_fd_s *fds, size_t nfds, int nonblock)
 	  if (fds[i].for_read)
 	    {
 	      struct reader_context_s *ctx = find_reader (fds[i].fd,1);
-
+	      
 	      if (!ctx)
 		TRACE_LOG1 ("error: no reader for FD 0x%x (ignored)",
 			    fds[i].fd);
@@ -1806,7 +1772,7 @@ _gpgme_io_select (struct io_select_fd_s *fds, size_t nfds, int nonblock)
 	  else if (fds[i].for_write)
 	    {
 	      struct writer_context_s *ctx = find_writer (fds[i].fd,1);
-
+              
 	      if (!ctx)
 		TRACE_LOG1 ("error: no writer for FD 0x%x (ignored)",
 			    fds[i].fd);
@@ -1829,7 +1795,7 @@ _gpgme_io_select (struct io_select_fd_s *fds, size_t nfds, int nonblock)
         }
     }
   TRACE_END (dbg_help, "]");
-  if (!any)
+  if (!any) 
     return TRACE_SYSRES (0);
 
   code = WaitForMultipleObjects (nwait, waitbuf, 0, nonblock ? 0 : 1000);
@@ -1869,7 +1835,7 @@ _gpgme_io_select (struct io_select_fd_s *fds, size_t nfds, int nonblock)
 	{
 	  int k;
 	  int j = handle_to_fd (waitbuf[i]);
-
+          
 	  TRACE_LOG1 ("WFMO invalid handle %d removed", j);
 	  for (k = 0 ; k < nfds; k++)
 	    {
@@ -1890,7 +1856,7 @@ _gpgme_io_select (struct io_select_fd_s *fds, size_t nfds, int nonblock)
       TRACE_LOG1 ("WFMO returned %d", code);
       count = -1;
     }
-
+  
   if (count > 0)
     {
       TRACE_SEQ (dbg_help, "select OK [ ");
@@ -1910,7 +1876,7 @@ _gpgme_io_select (struct io_select_fd_s *fds, size_t nfds, int nonblock)
       /* FIXME: Should determine a proper error code.  */
       gpg_err_set_errno (EIO);
     }
-
+  
   return TRACE_SYSRES (count);
 }
 
@@ -1960,7 +1926,7 @@ _gpgme_io_dup (int fd)
   newfd = new_fd();
   if (newfd == -1)
     return TRACE_SYSRES (-1);
-
+  
   fd_table[newfd].handle = fd_table[fd].handle;
   fd_table[newfd].socket = fd_table[fd].socket;
   fd_table[newfd].rvid = fd_table[fd].rvid;
@@ -2057,7 +2023,7 @@ _gpgme_io_socket (int domain, int type, int proto)
   fd = new_fd();
   if (fd == -1)
     return TRACE_SYSRES (-1);
-
+      
   res = socket (domain, type, proto);
   if (res == INVALID_SOCKET)
     {
@@ -2068,7 +2034,7 @@ _gpgme_io_socket (int domain, int type, int proto)
   fd_table[fd].socket = res;
 
   TRACE_SUC2 ("socket=0x%x (0x%x)", fd, fd_table[fd].socket);
-
+  
   return fd;
 }
 
@@ -2086,7 +2052,7 @@ _gpgme_io_connect (int fd, struct sockaddr *addr, int addrlen)
       gpg_err_set_errno (EBADF);
       return TRACE_SYSRES (-1);
     }
-
+    
   res = connect (fd_table[fd].socket, addr, addrlen);
   if (res)
     {
