@@ -29,33 +29,61 @@
 #include <assert.h>
 #include <errno.h>
 #include <stdint.h>
-#include <sys/time.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <unistd.h>
-#include <signal.h>
+#ifdef HAVE_SYS_TIME_H
+# include <sys/time.h>
+#endif
+#ifdef HAVE_SYS_TYPES_H
+# include <sys/types.h>
+#endif
+#ifdef HAVE_SYS_STAT_H
+# include <sys/stat.h>
+#endif
+#ifdef HAVE_UNISTD_H
+# include <unistd.h>
+#endif
 #include <fcntl.h>
-#include <windows.h>
-#include <shlobj.h>
 #include <io.h>
 
+#define _WIN32_IE 0x0400 /* Required for SHGetSpecialFolderPathA.  */
+
+/* We need to include the windows stuff here prior to shlobj.h so that
+   we get the right winsock version.  This is usually done in util.h
+   but that header also redefines some Windows functions which we need
+   to avoid unless having included shlobj.h.  */
+#include <winsock2.h>
+#include <ws2tcpip.h> 
+#include <windows.h>
+#include <shlobj.h>
+
 #include "util.h"
+#include "ath.h"
 #include "sema.h"
 #include "debug.h"
 
+
+#ifndef HAVE_W32CE_SYSTEM
+#define HAVE_ALLOW_SET_FOREGROUND_WINDOW 1
+#endif
+#ifndef F_OK
+# define F_OK 0
+#endif
+
+
 DEFINE_STATIC_LOCK (get_path_lock);
 
 
+#ifdef HAVE_ALLOW_SET_FOREGROUND_WINDOW
+
 #define RTLD_LAZY 0
 
-static __inline__ void *
+static GPG_ERR_INLINE void *
 dlopen (const char * name, int flag)
 {
   void * hd = LoadLibrary (name);
   return hd;
 }
 
-static __inline__ void *
+static GPG_ERR_INLINE void *
 dlsym (void * hd, const char * sym)
 {
   if (hd && sym)
@@ -68,7 +96,7 @@ dlsym (void * hd, const char * sym)
   return NULL;
 }
 
-static __inline__ int
+static GPG_ERR_INLINE int
 dlclose (void * hd)
 {
   if (hd)
@@ -78,6 +106,51 @@ dlclose (void * hd)
     }
   return -1;
 }  
+#endif /* HAVE_ALLOW_SET_FOREGROUND_WINDOW */
+
+void 
+_gpgme_allow_set_foreground_window (pid_t pid)
+{
+#ifdef HAVE_ALLOW_SET_FOREGROUND_WINDOW
+  static int initialized;
+  static BOOL (WINAPI * func)(DWORD);
+  void *handle;
+
+  if (!initialized)
+    {
+      /* Available since W2000; thus we dynload it.  */
+      initialized = 1;
+      handle = dlopen ("user32.dll", RTLD_LAZY);
+      if (handle)
+        {
+          func = dlsym (handle, "AllowSetForegroundWindow");
+          if (!func)
+            {
+              dlclose (handle);
+              handle = NULL;
+            }
+        }
+    }
+
+  if (!pid || pid == (pid_t)(-1))
+    {
+      TRACE1 (DEBUG_ENGINE, "gpgme:AllowSetForegroundWindow", 0,
+	      "no action for pid %d", (int)pid);
+    }
+  else if (func)
+    {
+      int rc = func (pid);
+      TRACE2 (DEBUG_ENGINE, "gpgme:AllowSetForegroundWindow", 0,
+	      "called for pid %d; result=%d", (int)pid, rc);
+
+    }
+  else
+    {
+      TRACE0 (DEBUG_ENGINE, "gpgme:AllowSetForegroundWindow", 0,
+	      "function not available");
+    }
+#endif /* HAVE_ALLOW_SET_FOREGROUND_WINDOW */
+}
 
 
 /* Return a string from the W32 Registry or NULL in case of error.
@@ -90,59 +163,64 @@ read_w32_registry_string (const char *root, const char *dir, const char *name)
   DWORD n1, nbytes, type;
   char *result = NULL;
 	
-  if ( !root )
+  if (!root)
     root_key = HKEY_CURRENT_USER;
-  else if ( !strcmp( root, "HKEY_CLASSES_ROOT" ) )
+  else if (!strcmp( root, "HKEY_CLASSES_ROOT"))
     root_key = HKEY_CLASSES_ROOT;
-  else if ( !strcmp( root, "HKEY_CURRENT_USER" ) )
+  else if (!strcmp( root, "HKEY_CURRENT_USER"))
     root_key = HKEY_CURRENT_USER;
-  else if ( !strcmp( root, "HKEY_LOCAL_MACHINE" ) )
+  else if (!strcmp( root, "HKEY_LOCAL_MACHINE"))
     root_key = HKEY_LOCAL_MACHINE;
-  else if ( !strcmp( root, "HKEY_USERS" ) )
+  else if (!strcmp( root, "HKEY_USERS"))
     root_key = HKEY_USERS;
-  else if ( !strcmp( root, "HKEY_PERFORMANCE_DATA" ) )
+  else if (!strcmp( root, "HKEY_PERFORMANCE_DATA"))
     root_key = HKEY_PERFORMANCE_DATA;
-  else if ( !strcmp( root, "HKEY_CURRENT_CONFIG" ) )
+  else if (!strcmp( root, "HKEY_CURRENT_CONFIG"))
     root_key = HKEY_CURRENT_CONFIG;
   else
     return NULL;
 	
-  if ( RegOpenKeyEx ( root_key, dir, 0, KEY_READ, &key_handle ) )
+  if (RegOpenKeyExA (root_key, dir, 0, KEY_READ, &key_handle))
     {
       if (root)
         return NULL; /* no need for a RegClose, so return direct */
       /* It seems to be common practise to fall back to HKLM. */
-      if (RegOpenKeyEx (HKEY_LOCAL_MACHINE, dir, 0, KEY_READ, &key_handle) )
+      if (RegOpenKeyExA (HKEY_LOCAL_MACHINE, dir, 0, KEY_READ, &key_handle))
         return NULL; /* still no need for a RegClose, so return direct */
     }
 
   nbytes = 1;
-  if ( RegQueryValueEx( key_handle, name, 0, NULL, NULL, &nbytes ) )
+  if (RegQueryValueExA (key_handle, name, 0, NULL, NULL, &nbytes))
     {
       if (root)
         goto leave;
       /* Try to fallback to HKLM also vor a missing value.  */
       RegCloseKey (key_handle);
-      if (RegOpenKeyEx (HKEY_LOCAL_MACHINE, dir, 0, KEY_READ, &key_handle) )
+      if (RegOpenKeyExA (HKEY_LOCAL_MACHINE, dir, 0, KEY_READ, &key_handle))
         return NULL; /* Nope.  */
-      if (RegQueryValueEx ( key_handle, name, 0, NULL, NULL, &nbytes))
+      if (RegQueryValueExA (key_handle, name, 0, NULL, NULL, &nbytes))
         goto leave;
     }
-  result = malloc ( (n1=nbytes+1) );
-  if ( !result )
+  n1 = nbytes + 1;
+  result = malloc (n1);
+  if (!result)
     goto leave;
-  if ( RegQueryValueEx ( key_handle, name, 0, &type, result, &n1 ) )
+  if (RegQueryValueExA (key_handle, name, 0, &type, (LPBYTE) result, &n1))
     {
-      free(result); result = NULL;
+      free (result);
+      result = NULL;
       goto leave;
     }
   result[nbytes] = 0; /* Make sure it is really a string.  */
+
+#ifndef HAVE_W32CE_SYSTEM
+  /* Windows CE does not have an environment.  */
   if (type == REG_EXPAND_SZ && strchr (result, '%')) 
     {
       char *tmp;
         
       n1 += 1000;
-      tmp = malloc (n1+1);
+      tmp = malloc (n1 + 1);
       if (!tmp)
         goto leave;
       nbytes = ExpandEnvironmentStrings (result, tmp, n1);
@@ -180,51 +258,14 @@ read_w32_registry_string (const char *root, const char *dir, const char *name)
           free (tmp);
         }
     }
+#endif
 
  leave:
-  RegCloseKey( key_handle );
+  RegCloseKey (key_handle);
   return result;
 }
 
 
-/* This is a helper function to load and run a Windows function from
-   either of one DLLs. */
-static HRESULT
-w32_shgetfolderpath (HWND a, int b, HANDLE c, DWORD d, LPSTR e)
-{
-  static int initialized;
-  static HRESULT (WINAPI * func)(HWND,int,HANDLE,DWORD,LPSTR);
-
-  if (!initialized)
-    {
-      static char *dllnames[] = { "shell32.dll", "shfolder.dll", NULL };
-      void *handle;
-      int i;
-
-      initialized = 1;
-
-      for (i=0, handle = NULL; !handle && dllnames[i]; i++)
-        {
-          handle = dlopen (dllnames[i], RTLD_LAZY);
-          if (handle)
-            {
-              func = dlsym (handle, "SHGetFolderPathA");
-              if (!func)
-                {
-                  dlclose (handle);
-                  handle = NULL;
-                }
-            }
-        }
-    }
-
-  if (func)
-    return func (a,b,c,d,e);
-  else
-    return -1;
-}
-
-
 #if 0
 static char *
 find_program_in_registry (const char *name)
@@ -286,7 +327,8 @@ find_program_at_standard_place (const char *name)
   char path[MAX_PATH];
   char *result = NULL;
       
-  if (w32_shgetfolderpath (NULL, CSIDL_PROGRAM_FILES, NULL, 0, path) >= 0) 
+  /* See http://wiki.tcl.tk/17492 for details on compatibility.  */
+  if (SHGetSpecialFolderPathA (NULL, path, CSIDL_PROGRAM_FILES, 0))
     {
       result = malloc (strlen (path) + 1 + strlen (name) + 1);
       if (result)
@@ -384,7 +426,7 @@ const char *
 _gpgme_get_uiserver_socket_path (void)
 {
   static char *socket_path;
-  char *homedir;
+  const char *homedir;
   const char name[] = "S.uiserver";
 
   if (socket_path)
@@ -433,52 +475,15 @@ _gpgme_get_conf_int (const char *key, int *value)
   return 1;
 }
 
-
-void 
-_gpgme_allow_set_foreground_window (pid_t pid)
-{
-  static int initialized;
-  static BOOL (WINAPI * func)(DWORD);
-  void *handle;
-
-  if (!initialized)
-    {
-      /* Available since W2000; thus we dynload it.  */
-      initialized = 1;
-      handle = dlopen ("user32.dll", RTLD_LAZY);
-      if (handle)
-        {
-          func = dlsym (handle, "AllowSetForegroundWindow");
-          if (!func)
-            {
-              dlclose (handle);
-              handle = NULL;
-            }
-        }
-    }
-
-  if (!pid || pid == (pid_t)(-1))
-    {
-      TRACE1 (DEBUG_ENGINE, "gpgme:AllowSetForegroundWindow", 0,
-	      "no action for pid %d", (int)pid);
-    }
-  else if (func)
-    {
-      int rc = func (pid);
-      TRACE2 (DEBUG_ENGINE, "gpgme:AllowSetForegroundWindow", 0,
-	      "called for pid %d; result=%d", (int)pid, rc);
-
-    }
-  else
-    {
-      TRACE0 (DEBUG_ENGINE, "gpgme:AllowSetForegroundWindow", 0,
-	      "function not available");
-    }
-
-}
-
-
 
+#ifdef HAVE_W32CE_SYSTEM
+int
+_gpgme_mkstemp (int *fd, char **name)
+{
+  return -1;
+}
+#else
+
 /* mkstemp extracted from libc/sysdeps/posix/tempname.c.  Copyright
    (C) 1991-1999, 2000, 2001, 2006 Free Software Foundation, Inc.
 
@@ -524,7 +529,7 @@ mkstemp (char *tmpl)
   len = strlen (tmpl);
   if (len < 6 || strcmp (&tmpl[len - 6], "XXXXXX"))
     {
-      errno = EINVAL;
+      gpg_err_set_errno (EINVAL);
       return -1;
     }
 
@@ -539,7 +544,7 @@ mkstemp (char *tmpl)
     random_time_bits = (((uint64_t)ft.dwHighDateTime << 32)
                         | (uint64_t)ft.dwLowDateTime);
   }
-  value += random_time_bits ^ getpid ();
+  value += random_time_bits ^ ath_self ();
 
   for (count = 0; count < attempts; value += 7777, ++count)
     {
@@ -561,7 +566,7 @@ mkstemp (char *tmpl)
       fd = open (tmpl, O_RDWR | O_CREAT | O_EXCL, S_IRUSR | S_IWUSR);
       if (fd >= 0)
 	{
-	  errno = save_errno;
+	  gpg_err_set_errno (save_errno);
 	  return fd;
 	}
       else if (errno != EEXIST)
@@ -569,7 +574,7 @@ mkstemp (char *tmpl)
     }
 
   /* We got out of the loop because we ran out of combinations to try.  */
-  errno = EEXIST;
+  gpg_err_set_errno (EEXIST);
   return -1;
 }
 
@@ -584,7 +589,7 @@ _gpgme_mkstemp (int *fd, char **name)
   *fd = -1;
   *name = NULL;
 
-  err = GetTempPath (MAX_PATH + 1, tmp);
+  err = GetTempPathA (MAX_PATH + 1, tmp);
   if (err == 0 || err > MAX_PATH + 1)
     strcpy (tmp,"c:\\windows\\temp");
   else
@@ -610,3 +615,25 @@ _gpgme_mkstemp (int *fd, char **name)
   *name = tmpname;
   return 0;
 }
+#endif
+
+
+
+#ifdef HAVE_W32CE_SYSTEM
+/* Return a malloced string with the replacement value for the
+   GPGME_DEBUG envvar.  Caller must release.  Returns NULL if not
+   set.  */
+char *
+_gpgme_w32ce_get_debug_envvar (void)
+{
+  char *tmp;
+
+  tmp = read_w32_registry_string (NULL, "\\Software\\GNU\\gpgme", "debug");
+  if (tmp && !*tmp)
+    {
+      free (tmp);
+      tmp = NULL;
+    }
+  return tmp;
+}
+#endif /*HAVE_W32CE_SYSTEM*/
