@@ -117,7 +117,7 @@ main (int argc, char **argv)
           gpgme_key_unref (key);
         }
     }
-  if (gpg_err_code (err) != GPG_ERR_EOF)
+  if (gpgme_err_code (err) != GPG_ERR_EOF)
     fail_if_err (err);
   err = gpgme_op_keylist_end (ctx);
   fail_if_err (err);
