@@ -1,22 +1,21 @@
 /* run-sign.c  - Helper to perform a sign operation
- * Copyright (C) 2009 g10 Code GmbH
- *
- * This file is part of GPGME.
- *
- * GPGME is free software; you can redistribute it and/or modify it
- * under the terms of the GNU Lesser General Public License as
- * published by the Free Software Foundation; either version 2.1 of
- * the License, or (at your option) any later version.
- *
- * GPGME is distributed in the hope that it will be useful, but
- * WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public
- * License along with this program; if not, see <https://gnu.org/licenses/>.
- * SPDX-License-Identifier: LGPL-2.1-or-later
- */
+   Copyright (C) 2009 g10 Code GmbH
+
+   This file is part of GPGME.
+ 
+   GPGME is free software; you can redistribute it and/or modify it
+   under the terms of the GNU Lesser General Public License as
+   published by the Free Software Foundation; either version 2.1 of
+   the License, or (at your option) any later version.
+   
+   GPGME is distributed in the hope that it will be useful, but
+   WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+   Lesser General Public License for more details.
+   
+   You should have received a copy of the GNU Lesser General Public
+   License along with this program; if not, see <http://www.gnu.org/licenses/>.
+*/
 
 /* We need to include config.h so that we know whether we are building
    with large file system (LFS) support. */
@@ -37,14 +36,6 @@
 
 static int verbose;
 
-static gpg_error_t
-status_cb (void *opaque, const char *keyword, const char *value)
-{
-  (void)opaque;
-  printf ("status_cb: %s %s\n", keyword, value);
-  return 0;
-}
-
 
 static void
 print_result (gpgme_sign_result_t result, gpgme_sig_mode_t type)
@@ -52,13 +43,11 @@ print_result (gpgme_sign_result_t result, gpgme_sig_mode_t type)
   gpgme_invalid_key_t invkey;
   gpgme_new_signature_t sig;
 
-  (void)type;
-
   for (invkey = result->invalid_signers; invkey; invkey = invkey->next)
-    printf ("Signing key `%s' not used: %s <%s>\n",
-            nonnull (invkey->fpr),
+    printf ("Signing key `%s' not used: %s <%s>\n", 
+            nonnull (invkey->fpr), 
             gpg_strerror (invkey->reason), gpg_strsource (invkey->reason));
-
+  
   for (sig = result->signatures; sig; sig = sig->next)
     {
       printf ("Key fingerprint: %s\n", nonnull (sig->fpr));
@@ -78,20 +67,16 @@ show_usage (int ex)
   fputs ("usage: " PGM " [options] FILE\n\n"
          "Options:\n"
          "  --verbose        run in verbose mode\n"
-         "  --status         print status lines from the backend\n"
          "  --openpgp        use the OpenPGP protocol (default)\n"
          "  --cms            use the CMS protocol\n"
          "  --uiserver       use the UI server\n"
-         "  --loopback       use a loopback pinentry\n"
          "  --key NAME       use key NAME for signing\n"
-         "  --sender MBOX    use MBOX as sender address\n"
-         "  --include-key-block  use this option with gpg\n"
          , stderr);
   exit (ex);
 }
 
 
-int
+int 
 main (int argc, char **argv)
 {
   int last_argc = -1;
@@ -102,11 +87,6 @@ main (int argc, char **argv)
   gpgme_sig_mode_t sigmode = GPGME_SIG_MODE_NORMAL;
   gpgme_data_t in, out;
   gpgme_sign_result_t result;
-  int print_status = 0;
-  int use_loopback = 0;
-  int include_key_block = 0;
-  const char *sender = NULL;
-  const char *s;
 
   if (argc)
     { argc--; argv++; }
@@ -126,11 +106,6 @@ main (int argc, char **argv)
           verbose = 1;
           argc--; argv++;
         }
-      else if (!strcmp (*argv, "--status"))
-        {
-          print_status = 1;
-          argc--; argv++;
-        }
       else if (!strcmp (*argv, "--openpgp"))
         {
           protocol = GPGME_PROTOCOL_OpenPGP;
@@ -154,29 +129,11 @@ main (int argc, char **argv)
           key_string = *argv;
           argc--; argv++;
         }
-      else if (!strcmp (*argv, "--sender"))
-        {
-          argc--; argv++;
-          if (!argc)
-            show_usage (1);
-          sender = *argv;
-          argc--; argv++;
-        }
-      else if (!strcmp (*argv, "--loopback"))
-        {
-          use_loopback = 1;
-          argc--; argv++;
-        }
-      else if (!strcmp (*argv, "--include-key-block"))
-        {
-          include_key_block = 1;
-          argc--; argv++;
-        }
       else if (!strncmp (*argv, "--", 2))
         show_usage (1);
-
-    }
-
+      
+    }          
+ 
   if (argc != 1)
     show_usage (1);
 
@@ -192,20 +149,14 @@ main (int argc, char **argv)
   fail_if_err (err);
   gpgme_set_protocol (ctx, protocol);
   gpgme_set_armor (ctx, 1);
-  if (print_status)
-    gpgme_set_status_cb (ctx, status_cb, NULL);
-  if (use_loopback)
-    gpgme_set_pinentry_mode (ctx, GPGME_PINENTRY_MODE_LOOPBACK);
 
   if (key_string)
     {
       gpgme_key_t akey;
-
+      
       err = gpgme_get_key (ctx, key_string, &akey, 1);
       if (err)
         {
-          fprintf (stderr, PGM ": get key '%s' failed: %s\n",
-                   key_string, gpg_strerror (err));
           exit (1);
         }
       err = gpgme_signers_add (ctx, akey);
@@ -213,23 +164,6 @@ main (int argc, char **argv)
       gpgme_key_unref (akey);
     }
 
-  if (sender)
-    {
-      err = gpgme_set_sender (ctx, sender);
-      fail_if_err (err);
-    }
-
-  if (include_key_block)
-    {
-      err = gpgme_set_ctx_flag (ctx, "include-key-block", "1");
-      if (err)
-        {
-          fprintf (stderr, PGM ": error setting include-key-block:  %s\n",
-                   gpgme_strerror (err));
-          exit (1);
-        }
-    }
-
   err = gpgme_data_new_from_file (&in, *argv, 1);
   if (err)
     {
@@ -251,9 +185,6 @@ main (int argc, char **argv)
       exit (1);
     }
 
-  if ((s = gpgme_get_ctx_flag (ctx, "redraw")) && *s)
-    fputs ("Screen redraw suggested\n", stdout);
-
   fputs ("Begin Output:\n", stdout);
   print_data (out);
   fputs ("End Output.\n", stdout);
